@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: __init__.py 11920 2020-02-12 07:55:26Z Lavender $
+# $Id: __init__.py 12000 2020-03-24 16:39:06Z Trista $
 #
 # Copyright (c) 2015 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -11,9 +11,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-# $Author: Lavender $ (last)
-# $Date: 2020-02-12 16:55:26 +0900 (週三, 12 二月 2020) $
-# $Revision: 11920 $
+# $Author: Trista $ (last)
+# $Date: 2020-03-25 00:39:06 +0800 (週三, 25 三月 2020) $
+# $Revision: 12000 $
 
 from __future__ import print_function
 
@@ -208,6 +208,19 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
     @param warns Whether to show warnings or not.
     @param printDetail Whether to print information or not.
     """
+    
+    args = sys.argv
+
+    # if the command is manage.py runserver, then print the SERVER_MODE
+    if (args[0].endswith('manage.py')) and (args[1]=='runserver'):
+        warns = True
+        printDetail = True
+        
+    # if the command is not manage.py runserver(ex: cms_dumpdata), do NOT print SERVER_MODE
+    else:
+        warns=False
+        printDetail = False
+    
     settingsPath = os.path.abspath(settingsDict['__file__'])
     projectPath = os.path.abspath(os.path.join(
         os.path.dirname(settingsPath), '..'))
