@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: __init__.py 11398 2019-03-24 20:35:15Z Kevin $
+# $Id: __init__.py 11475 2019-04-21 09:31:51Z Kevin $
 #
 # Copyright (c) 2017 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: Kevin $ (last)
-# $Date: 2019-03-25 04:35:15 +0800 (週一, 25 三月 2019) $
-# $Revision: 11398 $
+# $Date: 2019-04-21 17:31:51 +0800 (週日, 21 四月 2019) $
+# $Revision: 11475 $
 
 import datetime
 
@@ -198,4 +198,4 @@ class File:
 @contextmanager
 def open(file, mode='r', buffering=None, encoding=None,
          errors=None, newline=None, closefd=True):
-    return File(file, mode)
+    yield File(file, mode)
