@@ -4,6 +4,8 @@
 import datetime
 
 from django.db import models
+from django.db.models import F, Q
+from django.db import transaction
 from django.core import exceptions
 from django.contrib.auth import get_user_model
 from django.conf import settings
@@ -191,7 +193,64 @@ def bonus(self):
     return b.quantity
     
 User.bonus = bonus
+
+class BonusRecordManager(models.Manager):
+    @transaction.atomic
+    def recordUsingBonus(self, record, adjust=False):
+        bonusRecords = BonusRecord.objects.filter(
+            user=record.user.id, createdTime__lt=record.createdTime
+            ).exclude(id=record.id).order_by('createdTime') # old to new
+            
+        use = record.use
+        
+        if record.adjust < 0: # 調整紅利為負，表示扣除
+            use = use - record.adjust
+        
+        for r in bonusRecords:
+            if use == 0: # 已記錄完畢
+                break
+            else:
+                canUsingBonus = r.canUsingBonus
+                if canUsingBonus > 0:
+                    if canUsingBonus >= use:
+                        r.usedBonus = r.usedBonus + use
+                        use = use - use # 0
+                    else:
+                        use = use - canUsingBonus
+                        r.usedBonus = r.usedBonus + canUsingBonus # canUsingBonus will be 0
+                        
+                    r.save()
+                
+        if use != 0:
+            if adjust:
+                # 用於紀錄對不上時(如使用 admin 自行修改 bonus 而沒有被 record 處理過的)，這部分用於 migrate 時，
+                # 調整當筆紀錄之前的紀錄並扣除已使用的 bonus
+                # 屬於紀錄補正用
+                r = BonusRecord.objects.create(
+                    user_id=int(record.user.id),
+                    adjust=use,
+                    usedBonus=use,
+                )
+                r.createdTime = record.createdTime
+                r.save()
+            else:
+                raise RuntimeError("Not enough bonus to deduct: %s" % record)
+        
     
+    @transaction.atomic
+    def createBonusRecord(self, user, order, use=0, receive=0, adjust=0):
+        record = BonusRecord.objects.create(
+            user=user,
+            order=order,
+            use=use,
+            receive=receive,
+            adjust=adjust,
+        )
+        
+        # recordUsingBonus
+        self.recordUsingBonus(record)
+        return record
+   
 class BonusRecord(models.Model):
     user = models.ForeignKey(
         User,
@@ -213,8 +272,51 @@ class BonusRecord(models.Model):
     adjust = models.IntegerField(
         verbose_name=_("Adjust"), default=0)
         
+    usedBonus = models.IntegerField(
+        verbose_name=_("Used bonus"), default=0)
+        
+    expiredBonus = models.IntegerField(
+        verbose_name=_("Expired bonus"), default=0)
+        
     createdTime = models.DateTimeField(_("Created Time"), auto_now_add=True)
     
+    objects = BonusRecordManager()
+    
+    @property
+    def canUsingBonus(self):
+        return self.receive + self.adjust - self.usedBonus - self.expiredBonus
+            
+    @property
+    def expiredTime(self):
+        config = BonusSettings.objects.get()
+        if config.expiryDays >= 0:
+            return self.createdTime + datetime.timedelta(days=config.expiryDays)
+        else:
+            return None # no limit
+            
+    def updateExpiredBonus(self):
+        config = BonusSettings.objects.get()
+        b = Bonus.objects.get(user=self.user)
+        now = timezone.localtime(timezone.now())
+        if config.expiryDays >= 0:
+            if self.expiredTime <= now and self.expiredBonus == 0: # 還沒紀錄過過期點數
+                canUsingBonus = self.canUsingBonus
+                if canUsingBonus > 0:
+                    self.expiredBonus = canUsingBonus
+                    self.save()
+                    
+                    if b.quantity - self.expiredBonus >= 0:
+                        b.quantity = b.quantity - self.expiredBonus
+                    else:
+                        # 過期點數沒得扣除，因此設為 0，原因出自於 database 自行更改 bonus 值或者在此bonus過期機制版本
+                        # 更新前有 bug 時修改過 bonus 所造成的
+                        b.quantity = 0
+                    b.save()           
+    
+    def __str__(self):
+        return "<BonusRecord: user: %s order: %s createdTime: %s>" % (self.user, self.order, self.createdTime)
+        
+    
 @property
 def hasReceiveBonus(self):
     return BonusRecord.objects.filter(order=self).exists()
@@ -288,6 +390,9 @@ class BonusSettings(SingletonModel):
         related_name='bonusExcludeRange'
     )
     
+    expiryDays = models.IntegerField(
+        verbose_name=_("Expiry days(-1 is no limit)"), default=-1, validators=[MinValueValidator(-1),],)
+    
 # https://code.nuwainfo.com/trac/Mercurius/ticket/3301#ticket
 from oscar.apps.customer.managers import CommunicationTypeManager
 CommunicationEventType = get_model('customer', 'CommunicationEventType')
